#include "Config.h"

#include <cstring>
#include <streambuf>
#include <string>

#if defined(_DFF_ON_WINDOWS_)
# include <direct.h>
# include <shellapi.h>
# include "Shlwapi.h"
#else
# include <unistd.h>
# include <stdlib.h>
#endif

#include "Exception.h"
#include "ExceptionUtils.h"
#include "FilesystemUtils.h"
#include "SystemUtils.h"


#define TEMP_DIR_ENV_VARIABLE_NAME "DINAMICA_TEMP_DIR"


namespace DFF {


namespace {


std::string getUserTemporaryDirectory() {
  return DFF::SystemUtils::getEnvironmentVariableValue( 
      TEMP_DIR_ENV_VARIABLE_NAME );
}

std::string getOSTemporaryDirectory() {
#ifdef _DFF_ON_WINDOWS_
  char tmpBuffer[ 1024 ] = { 0 };
  ::GetTempPath( 1024, tmpBuffer );

  return std::string( tmpBuffer );
#else
  std::string tmpBuffer = 
    DFF::SystemUtils::getEnvironmentVariableValue( "TMPDIR" );

  if ( tmpBuffer.empty() )
    tmpBuffer = "/tmp";

  return tmpBuffer;
#endif
}

std::string getSystemTemporaryDirectory() {
  const auto userDirectory( getUserTemporaryDirectory() );
  const auto tmpDirectory( getOSTemporaryDirectory() );
  return userDirectory.empty() ? tmpDirectory : userDirectory;
}


} // namespace ''


#if defined(_DFF_ON_WINDOWS_)
# define getCurrentDirectory _getcwd
#else
# define MAX_PATH 255
# define getCurrentDirectory getcwd
#endif

std::string FilesystemUtils::getCompleteTemporaryPath( const std::string& baseName ) {
#ifdef _DFF_ON_WINDOWS_
  return ( getSystemTemporaryDirectory() + "\\" + baseName );
#else
  return ( getSystemTemporaryDirectory() + "/" + baseName );
#endif
}

std::string FilesystemUtils::getTemporaryFilename( const std::string& extension ) {
#ifdef _DFF_ON_WINDOWS_
  char tempFilenameBuffer[ 1024 ] = { 0 };
  if ( ::GetTempFileName( getSystemTemporaryDirectory().c_str(),
    "EGO", 0, tempFilenameBuffer ) == 0 ) {

    std::string errMsg;
    SystemUtils::getLastSystemError( errMsg );

    throwException( DFF::Exception, "Failed to get temporary file: " <<
      errMsg );
  }

  return std::string( tempFilenameBuffer ) + extension;
#else
  std::string tempFilename( getSystemTemporaryDirectory() + 
    "/dinamica-ego-r-XXXXXX" );

  char* tempFilenameBuffer = strdup( tempFilename.c_str() );
  const int fileDescriptor = mkstemp( tempFilenameBuffer );
  if ( fileDescriptor != -1 ) {
    std::string resultString( tempFilenameBuffer );
    free( tempFilenameBuffer );
    close( fileDescriptor );

    return resultString + extension;
  } else {
    std::string errMsg;
    SystemUtils::getLastSystemError( errMsg );

    throwException( DFF::Exception, "Failed to get temporary file: " <<
      errMsg );
  }

  // Nunca deve alcançar.
  return std::string();
#endif
}


} // namespace DFF

